package jumpstart.max.business.domain.security;

import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.PostLoad;
import javax.persistence.PrePersist;
import javax.persistence.PreRemove;
import javax.persistence.PreUpdate;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.persistence.UniqueConstraint;

import jumpstart.max.business.commons.exception.BusinessException;
import jumpstart.max.business.commons.exception.ReferenceNotResettableException;
import jumpstart.max.business.commons.exception.ReferencesWrongSubsetException;
import jumpstart.max.business.commons.exception.ValueRequiredException;
import jumpstart.max.business.domain.base.BaseEntityAutoGenId;
import jumpstart.max.business.domain.reference.Code;
import jumpstart.max.business.domain.reference.CodeGroup;

/**
 * The UserRole entity.
 * 
 * @version Revision: 0.1, Date: 10/06/2006
 */
@Entity
@Table(name = "user_role", uniqueConstraints = { @UniqueConstraint(columnNames = { "user_id", "role_id" }) })
@SuppressWarnings("serial")
public class UserRole extends BaseEntityAutoGenId {
//	static private Log LOG = LogFactory.getLog(UserRole.class);
	static public String STATUS_ACTIVE = "uro_active"; 
	static public String STATUS_INACTIVE = "uro_inactive"; 

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "user_id", nullable = false)
	protected User user;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "role_id", nullable = false)
	protected Role role;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "status_code", nullable = false)
	protected Code status;

	@Transient
	protected transient User _loaded_user = null;

	@Transient
	protected transient Role _loaded_role = null;

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("UserRole: [");
		buf.append("id=" + id + ", ");
		buf.append("user=" + toStringLazy(user, "getId") + ", ");
		buf.append("role=" + toStringLazy(role, "getId") + ", ");
		buf.append("status=" + toStringLazy(status, "getId") + ", ");
		buf.append("version=" + version);
		buf.append("]");
		return buf.toString();
	}

	@PrePersist
	void prePersistOfUserRole() throws BusinessException {
		validateUserRole();
	}

	@PostLoad
	void postLoadOfUserRole() {
		_loaded_user = user;
		_loaded_role = role;
	}

	@PreUpdate
	void preUpdateOfUserRole() throws BusinessException {

		// user can be set only once because it's a reference to the parent

		if (!(user.getId().equals(_loaded_user.getId()))) {
			throw new ReferenceNotResettableException(this, "UserRole_user", _loaded_user.getId(), user.getId());
		}

		// role can be set only once because it's a reference to the parent

		if (!(role.getId().equals(_loaded_role.getId()))) {
			throw new ReferenceNotResettableException(this, "UserRole_role", _loaded_role.getId(), role.getId());
		}
		validateUserRole();
	}

	@PreRemove
	void preRemoveOfUserRole() throws BusinessException {
		// Check business rules here, eg.
		// if (entity.getParts().size() > 0) {
		// throw new CannotDeleteIsNotEmptyException(this, id, "Part");
		// }

		// There's no need to remove me from the parent(s) - that's the caller's
		// responsibility (and for performance, it might not bother)
	}

	public void validateUserRole() throws BusinessException {

		// Validate syntax...

		if (user == null) {
			throw new ValueRequiredException(this, "UserRole_user");
		}

		if (role == null) {
			throw new ValueRequiredException(this, "UserRole_role");
		}

		if (status == null) {
			throw new ValueRequiredException(this, "UserRole_status");
		}

		// Validate semantics...

		if (!(status.getCodeGroup().getId().equals(CodeGroup.ID_USERROLE_STATUS))) {
			throw new ReferencesWrongSubsetException(this, "UserRole_status", "", "CodeGroup", CodeGroup.ID_ROLE_STATUS
					.toString(), status.getCodeGroup().getId().toString());
		}

		if (!(status.getStatus().getId().equals(Code.ID_CODE_STATUS_ACTIVE))) {
			throw new ReferencesWrongSubsetException(this, "UserRole_status", "", "Status",
					Code.ID_CODE_STATUS_ACTIVE.toString(), status.getStatus().getId().toString());
		}

	}

	public Code getStatus() {
		return status;
	}

	public void setStatus(Code status) {
		this.status = status;
	}

	public User getUser() {
		return user;
	}

	/**
	 * Establishes my side of the bi-directional relationship with my parent,
	 * User. If detached, you MUST also call user.addUserRole(userRole). If
	 * attached, you can choose not to call user.addUserRole(userRole) for
	 * performance reasons, but be aware that as a consequence you would also
	 * miss out on any business rule validaton in that method.
	 */
	public void setUser(User user) {
		this.user = user;
	}

	public Role getRole() {
		return role;
	}

	/**
	 * Establishes my side of the bi-directional relationship with my parent,
	 * Role. If detached, you MUST also call role.addUserRole(userRole). If
	 * attached, you can choose not to call role.addUserRole(userRole) for
	 * performance reasons, but be aware that as a consequence you would also
	 * miss out on any business rule validaton in that method.
	 */
	public void setRole(Role role) {
		this.role = role;
	}
}
