package jumpstart.max.web.pages.base;

import jumpstart.max.business.commons.exception.CannotDeleteIsReferencedException;
import jumpstart.max.business.commons.exception.DuplicateAlternateKeyException;
import jumpstart.max.business.commons.exception.DuplicatePrimaryKeyException;
import jumpstart.max.business.commons.exception.OptimisticLockException;
import jumpstart.max.business.commons.exception.ServiceException;
import jumpstart.max.business.commons.exception.UnexpectedException;
import jumpstart.max.business.commons.interpreter.BizServicesExceptionInterpreter;
import jumpstart.max.util.ExceptionFormatter;
import jumpstart.max.util.StringUtil;
import jumpstart.max.web.state.BizServicesLocator;
import jumpstart.max.web.state.Visit;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.annotations.Bean;
import org.apache.tapestry.annotations.InjectState;
import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.html.BasePage;
import org.apache.tapestry.valid.ValidationConstraint;
import org.apache.tapestry.valid.ValidationDelegate;
import org.joda.time.Instant;

/**
 * All pages in the application can be descendents of this page.
 * Pages needing protection from users who are not logged in should
 * extend ProtectedBasePage.
 *
 */
public abstract class SimpleBasePage extends BasePage {
	static public final Log LOG = LogFactory.getLog(SimpleBasePage.class);
	
	private BizServicesExceptionInterpreter _interpreter = new BizServicesExceptionInterpreter();

	@InjectState("bizServicesLocator")
	public abstract BizServicesLocator getBizServicesLocator();

	@InjectState("visit")
	// Can't call it getVisit() because it conflicts with a deprecated IPage method
	public abstract Visit getMyVisit();

	@Bean
	public abstract ValidationDelegate getValidationDelegate();

	// @InitialValue("")
	public abstract void setInfoMessage(String value);

	// @InitialValue("")
	public abstract void setWarningMessage(String value);

	protected void recordInfo(String message) {
		setInfoMessage(message);
	}

	protected void recordWarning(String message) {
		setWarningMessage(message);
	}

	protected void recordError(String message) {
		ValidationDelegate delegate = getValidationDelegate();
		delegate.setFormComponent(null);
		delegate.record(message, ValidationConstraint.CONSISTENCY);
	}

	protected void recordError(String message, String fieldName) {
		ValidationDelegate delegate = getValidationDelegate();
		delegate.setFormComponent((IFormComponent) getComponent(fieldName));
		delegate.record(message, null);
	}

	protected boolean hasErrors() {
		boolean hasErrors = getValidationDelegate().getHasErrors();
		return hasErrors;
	}

	protected boolean isEmpty(String s) {
		return StringUtil.isEmpty(s);
	}
	
	protected void handleBizServicesExceptionForCreate(Exception e) {
		ServiceException x = _interpreter.interpret(e);

		if (x instanceof DuplicatePrimaryKeyException) {
			recordError(getMessages().getMessage("create_failed_duplicate_primary_key"));
		}
		else if (x instanceof DuplicateAlternateKeyException) {
			DuplicateAlternateKeyException d = (DuplicateAlternateKeyException) x;
			recordError(getMessages().format("create_failed_duplicate_alternate_key", d.getTechnicalMessageText()));
		}
		else if (x instanceof UnexpectedException) {
			recordAndLogUnexpectedException("Create entity", e, x);
		}
		else {
			recordError(x.getMessage());
		}

	}

	protected void handleBizServicesExceptionForAdd(Exception e) {
		ServiceException x = _interpreter.interpret(e);

		if (x instanceof OptimisticLockException) {
			recordError(getMessages().getMessage("add_failed_optimistic_lock"));
		}
		else if (x instanceof DuplicatePrimaryKeyException) {
			recordError(getMessages().getMessage("add_failed_duplicate_primary_key"));
		}
		else if (x instanceof DuplicateAlternateKeyException) {
			DuplicateAlternateKeyException d = (DuplicateAlternateKeyException) x;
			recordError(getMessages().format("add_failed_duplicate_alternate_key", d.getTechnicalMessageText()));
		}
		else if (x instanceof UnexpectedException) {
			recordAndLogUnexpectedException("Add entity", e, x);
		}
		else {
			recordError(x.getMessage());
		}

	}

	protected void handleBizServicesExceptionForChange(Exception e) {
		ServiceException x = _interpreter.interpret(e);

		if (x instanceof OptimisticLockException) {
			recordError(getMessages().getMessage("change_failed_optimistic_lock"));
		}
		else if (x instanceof DuplicateAlternateKeyException) {
			DuplicateAlternateKeyException d = (DuplicateAlternateKeyException) x;
			recordError(getMessages().format("change_failed_duplicate_alternate_key", d.getTechnicalMessageText()));
		}
		else if (x instanceof UnexpectedException) {
			recordAndLogUnexpectedException("Change entity", e, x);
		}
		else {
			recordError(x.getMessage());
		}

	}

	protected void handleBizServicesExceptionForRemove(Exception e) {
		ServiceException x = _interpreter.interpret(e);

		if (x instanceof OptimisticLockException) {
			recordError(getMessages().getMessage("remove_failed_optimistic_lock"));
		}
		else if (x instanceof CannotDeleteIsReferencedException) {
			CannotDeleteIsReferencedException c = (CannotDeleteIsReferencedException) x;
			recordError(getMessages().format("remove_failed_is_referenced", new Object[] { c.getReferencedByEntityName() }));
		}
		else if (x instanceof UnexpectedException) {
			recordAndLogUnexpectedException("Remove entity", e, x);
		}
		else {
			recordError(x.getMessage());
		}

	}

	protected void handleBizServicesExceptionForDelete(Exception e) {
		ServiceException x = _interpreter.interpret(e);

		if (x instanceof OptimisticLockException) {
			recordError(getMessages().getMessage("delete_failed_optimistic_lock"));
		}
		else if (x instanceof CannotDeleteIsReferencedException) {
			CannotDeleteIsReferencedException c = (CannotDeleteIsReferencedException) x;
			recordError(getMessages().format("delete_failed_is_referenced", new Object[] { c.getReferencedByEntityName() }));
		}
		else if (x instanceof UnexpectedException) {
			recordAndLogUnexpectedException("Delete entity", e, x);
		}
		else {
			recordError(x.getMessage());
		}

	}

	protected void recordAndLogUnexpectedException(String contextDescription, Exception e, Throwable rootCause) {
		Instant instant = new Instant();
		LOG.error("Business services layer returned unexpected exception.  Reference code is \"C" + instant + "\".  Operation was: "
				+ contextDescription + ".");
		LOG.error("** Exception was: " + e.getMessage());
		LOG.error("** Root cause was: " + rootCause.getMessage());
		LOG.error("** Stack trace follows...");
		LOG.error(ExceptionFormatter.toAString(e));
		recordError(getMessages().format("application_unexpected_exception", instant));
	}

}
